<?php
/**
 * For more information about WP_List_Table http://codex.wordpress.org/Class_Reference/WP_List_Table
 */
class WC_Custom_Status_Table extends WP_List_Table {
    protected $data;
    function __construct(){
        parent::__construct( array(
          'singular'  => __( 'wc_custom_status_table', 'woocommerce_status_actions' ),     //singular name of the listed records
          'plural'    => __( 'wc_custom_status_tables', 'woocommerce_status_actions' ),   //plural name of the listed records
          'ajax'      => false        //does this table support ajax?
      ) );
    }


    function no_items() {
        _e( 'No Order Statuses found', 'woocommerce_status_actions' );
    }

    function column_default( $item, $column_name ) {
        switch ( $column_name ) {
            case 'col_status_name':
            case 'col_status_label':
            case 'col_status_icon':
            case 'col_action_icon':
            case 'col_sends_email':
            case 'display_in_reports':
            case 'item_editing':
            case 'automatic_trigger':
            case 'number_of_orders':
                return $item[$column_name];
            default:
                return print_r( $item, true ); //Show the whole array for troubleshooting purposes
        }
    }
     function get_sortable_columns() {
        $sortable_columns = array(
            'col_status_name' => array('status_name', true)
        );
        return $sortable_columns;
    }

    /**
     * Defines the table's columns
     * @return [array] [key => title for table headers]
     */
    function get_columns(){
        return array(
            'cb' => '<input type="checkbox" />',
            'col_status_name' => __('Name','woocommerce_status_actions'),
            'col_status_label' => __('Label','woocommerce_status_actions'),
            'col_status_icon' => __('Icon','woocommerce_status_actions'),
            'col_action_icon' => __('Action','woocommerce_status_actions'),
            'col_sends_email' => __('Email','woocommerce_status_actions'),
            'display_in_reports' => __('Reports','woocommerce_status_actions'),
            'item_editing' => __('Editing','woocommerce_status_actions'),
            'automatic_trigger' => __('Trigger','woocommerce_status_actions'),
            'number_of_orders' => __('Orders','woocommerce_status_actions'),
        );
    }

    function column_cb($item) {        
        return sprintf(
            '<input type="checkbox" name="status_id[]" value="%s" />', $item['id']
            );
    }

    function column_col_status_name( $item ) {
        //edit link
        $editlink  = admin_url().'admin.php?page=woocommerce_status_and_actions&tab=edit_status&status_id='.(int)$item['id'];
        if(   defined( 'WC_VERSION') && floatval(WC_VERSION) > 2.1 ){
            $deletelink = admin_url().'admin.php?page=woocommerce_status_and_actions&tab=delete_status&status_id='.(int)$item['id'];
        }else{
            $deletelink = admin_url().'admin.php?page=woocommerce_status_and_actions&tab=delete_status&status_id='.(int)$item['id'];
        }

        $name = '<strong><a class="row-title" href="'.$editlink.'" title="Edit “'.stripslashes($item['status_name']).'”">'.stripslashes($item['status_name']).'</a></strong>';
        $actions = array(
            'edit'      => sprintf('<a href="%s">Edit</a>',$editlink),
            'delete' => sprintf('<a href="%s">Delete Permanently</a>',$deletelink)
        );
        return sprintf('%1$s %2$s', $name, $this->row_actions($actions) );
    }

    function column_col_status_label($item){
        return stripslashes($item['status_label']);
    }
    function column_col_status_icon( $item ) {
        // Style!
        $style = '';
        $class = '';
        if( empty( $item['status_color_type'] ) || $item['status_color_type'] == '0' ) {
            $style = 'style="color:'. $item['status_color'] .'; border: 1px solid '. $item['status_color'] .';"';
            $class = 'new_status_icon';
        } else if( $item['status_color_type'] == '1' ){
            $style = 'style="background-color: '.$item['status_color'].';"';
            $class = 'old_status_icon';
        }
        $html = '';
        if(empty($item['status_label'])){
            $item['status_label'] = $item['status_slug'];
            $this->update_status_label($item['id'], $item['status_slug']);
        }
        if( empty( $item['status_color_type'] ) || $item['status_color_type'] == '0' || $item['status_color_type'] == '1' ) {
            $html = '<span '.$style.'  class="woocommerce_status_font_icon woocommerce_status_actions_'.$item['status_icon'].' tips '.$class.'" data-tip="'.$item['status_name'].'" alt="'.$item['status_label'].'"></span></td>';
        } else if( $item['status_color_type'] == '2' ){
            $html = '<strong class="status_orders_page status_text" style="background-color:'.$item['status_color'].';">'.strtolower($item['status_label']).'</strong>';
        } else {
            $html = '<strong class="status_orders_page status_line_text" style="color:'.$item['status_color'].';">'.strtolower($item['status_label']).'</strong>';
        }
        return $html;
    }

    function column_col_sends_email( $item ) {
        if($item['sends_email'] == '1' && $item['email_to'] == 'both') {
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . esc_attr__( 'Administrator & Customer', 'woocommerce' ) . '"></span>';
        } else if($item['sends_email'] == '1' && $item['email_to'] == 'customer') {
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . esc_attr__( 'Customer', 'woocommerce' ) . '"></span>';
        } else if($item['sends_email'] == '1' && $item['email_to'] == 'admin') {
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . esc_attr__( 'Administrator', 'woocommerce' ) . '"></span>';
        } else if($item['sends_email'] == '1' && $item['email_to'] == 'custom') {
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . $item['custom_email_address'] . '"></span>';
        } else{
            return '<span style="color: #999;" class="woocommerce-csa-cross-icon tips status-table-icons" data-tip="' . esc_attr__( 'No', 'woocommerce' ) . '"></span>';
        }
    }
    function column_col_action_icon( $item ) {
        $item['status_name'] = ucfirst($item['status_name']);        
        $action_icon = empty($item['action_icon'])?'e372':$item['action_icon'];
        return '<a class="button default-action-icon-selector woocommerce_status_action_font_icon woocommerce_status_actions_'.$action_icon.'" href="javascript:;">Icon Preview</a>';
    }
    function column_number_of_orders( $item ) {
        global $wpdb;
        $slug = $item['status_slug'];

        $sql = "SELECT COUNT(DISTINCT ID) FROM {$wpdb->posts} WHERE post_status = 'wc-{$slug}' ";
        $count = $wpdb->get_var($sql);
        return $count;
    }

    function column_display_in_reports( $item ) {
        if($item['display_in_reports'] == '1'){
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . esc_attr__( 'Included In Reports', 'woocommerce' ) . '"></span>';
        } else {
            return '<span style="color: #999;" class="woocommerce-csa-cross-icon tips status-table-icons" data-tip="' . esc_attr__( 'Not Included In Reports', 'woocommerce' ) . '"></span>';
        }
    }
    function column_item_editing( $item ) {
        
        if(wcos_get_meta( $item['id'] , 'order_is_editable', true) == 'yes'){
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . esc_attr__( 'Item Editing Enabled', 'woocommerce' ) . '"></span>';
        } else {
            return '<span style="color: #999;" class="woocommerce-csa-cross-icon tips status-table-icons" data-tip="' . esc_attr__( 'Item Editing Disabled', 'woocommerce' ) . '"></span>';
        }
    }
    function column_automatic_trigger( $item ) {
        
        if(wcos_get_meta( $item['id'] , 'automatic_trigger', true) == '1'){
            return '<span style="color: #ad74a2;" class="woocommerce-csa-tick-icon tips status-table-icons" data-tip="' . esc_attr__( 'Automatic Trigger To', 'woocommerce' ) . ' ' . ucwords(wcos_get_meta( $item['id'] , 'triggered_status', true)) . ' ' . esc_attr__( 'After', 'woocommerce' ) . ' ' . wcos_get_meta( $item['id'] , 'time_period', true) . ' ' . ucfirst(wcos_get_meta( $item['id'] , 'time_period_type', true)) . '"></span>';
        } else {
            return '<span style="color: #999;" class="woocommerce-csa-cross-icon tips status-table-icons" data-tip="' . esc_attr__( 'Automatic Trigger Disabled', 'woocommerce' ) . '"></span>';
        }
    }

    /**
     * Prepares the data/rows to be displayed in the table
     * @return void
     */
    function prepare_items(){
        global $wpdb, $_wp_column_headers;
        $screen = get_current_screen();

        $table_name = $wpdb->prefix . 'woocommerce_order_status_action';
        /* -- Preparing your query -- */
        $query = "SELECT * FROM {$table_name}";

        /* -- Ordering parameters -- */
        //Parameters that are going to be used to order the result
        $orderby = !empty($_GET["orderby"]) ? mysql_real_escape_string($_GET["orderby"]) : 'ASC';
        $order = !empty($_GET["order"]) ? mysql_real_escape_string($_GET["order"]) : '';
        if(!empty($orderby) & !empty($order)){ $query.=' ORDER BY '.$orderby.' '.$order; }

        /* -- Pagination parameters -- */
        $result = $wpdb->get_results($query, ARRAY_A);        

        /* -- Register the Columns -- */
        $this->_column_headers = array(
            $this->get_columns(),       // columns
            array(),            // hidden
            $this->get_sortable_columns(),  // sortable
        );

        $current_page = $this->get_pagenum();
    
        $user = get_current_user_id();
        $screen = get_current_screen();
        $option = $screen->get_option('per_page', 'option');
        $per_page = get_user_meta($user, $option, true);
        if ( empty ( $per_page) || $per_page < 1 ) {
            $per_page = $screen->get_option( 'per_page', 'default' );
        }

        $total_items = count( $result );
          // only ncessary because we have sample data
          $found_data = array_slice( $result,( ( $current_page-1 )* $per_page ), $per_page );
          $this->set_pagination_args( array(
            'total_items'   => $total_items,                  //WE have to calculate the total number of items
            'per_page' => $per_page                     //WE have to determine how many items to show on a page
          ) );
          $this->items = $found_data;
    }

    
    /**
     * Auto generate the bulk actions list
     * @return [array] [key => label for the bulk action selectbox]
     */
    function get_bulk_actions() {
        $actions = array(
            'delete'    => 'Delete Permanently'
        );
        return $actions;
    }

    function update_status_label($id, $slug){
        global $wpdb;
        $table_name = $wpdb->prefix . 'woocommerce_order_status_action';
        $wpdb->update($table_name, array('status_label'=>$slug), array('id'=>$id));
    }
}
?>
