<?php
/**
 * Admin Dashboard
*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'WC_CSA_Dashboard' ) ) :

/**
 * WC_CSA_Dashboard Class
 */
class WC_CSA_Dashboard {

	/**
	 * Hook in tabs.
	 */
	public function __construct() {
		// Only hook in admin parts if the user has admin access
		if ( current_user_can( 'view_woocommerce_reports' ) || current_user_can( 'manage_woocommerce' ) || current_user_can( 'publish_shop_orders' ) ) {
			add_action( 'wp_dashboard_setup', array( $this, 'init' ) );
		}
	}

	/**
	 * Init dashboard widgets
	 */
	public function init() {
		wp_add_dashboard_widget( 'woocommerce_csa_dashboard_status', __( 'WooCommerce Status', 'woocommerce' ), array( $this, 'status_widget' ) );
	}

	/**
	 * Show status widget
	 */
	public function status_widget() {
		global $wpdb;

		$show_in_widget = $wpdb->get_results("SELECT * FROM {$wpdb->wc_osa} osa 
			INNER JOIN {$wpdb->wc_osmeta} meta ON ( osa.id = meta.wc_os_id AND meta.meta_key = 'dashboard_widget' ) 
			WHERE meta.meta_value = 'yes' 
			");

		// Counts
		$status_count = array();

		if($show_in_widget){
			foreach ($show_in_widget as $status) {
				foreach ( wc_get_order_types( 'order-count' ) as $type ) {
					$counts           = (array) wp_count_posts( $type );
					if(!isset($status_count[$status->id])){
						$status_count[$status->id] = $status;
						$status_count[$status->id]->count = 0;
					}

					$status_count[$status->id]->count   += isset( $counts['wc-' . $status->status_slug] ) ? $counts['wc-' . $status->status_slug] : 0;
				}				
			}
		}
		if( !empty($status_count) ){
			echo '<ul class="wc_status_list">';
			$i = 0;
			foreach ($status_count as $id => $st) { $i++;
				$class= '';
				if( $i == count($status_count) && $i%2 > 0)
					$class= 'last';
			?>
			<li class="<?php echo $st->status_slug; ?>-orders <?php echo $class; ?>">
				<a href="<?php echo admin_url( 'edit.php?post_status=wc-'.$st->status_slug.'&post_type=shop_order' ); ?>">
					<?php 
					$c = sprintf( _n( "<strong>%s order</strong>", "<strong>%s orders</strong>", $st->count, 'woocommerce' ), $st->count ) . ' ' . $st->status_label;
					echo strtolower($c);
					?>
				</a>
			</li>
			<?php				
			}
			echo '</ul>';
			echo '<style>';
			foreach ($status_count as $id => $st) {
				echo '#woocommerce_csa_dashboard_status .wc_status_list li.'.$st->status_slug.'-orders a::before{';
					echo 'border: solid 2px '.$st->status_color.';';
					echo 'color: '.$st->status_color.';';
					echo 'content: "\\'.$st->status_icon.'";';
				echo '}';
			}
			echo '</style>';
		}
	}


}

endif;

return new WC_CSA_Dashboard();
