<?php

class WC_Custom_Status_Plugin {

    /**
     * Hooks the plugin's functions
     */
    function __construct() {

        global $woocommerce;
        // Adds the Status and Actions to the menu
        add_action('woocommerce_init',array(&$this,'after_woocommerce_init'));
        
        // Custom work when an order status changes to one of our statuses
        add_action('woocommerce_order_status_changed',array(&$this,'process_order_status'),0,3);
        // Add columns to the orders list
        add_filter('manage_shop_order_posts_custom_column', array(&$this,'payment_column_manage'),11);
        add_filter('woocommerce_admin_order_actions',array(&$this,'action_column_manage'),11,2);
        
        // Handles ajax request for deleting statuses
        add_action( 'wp_ajax_delete_custom_status', array( &$this, 'woocommerce_status_actions_delete') );


        // Cancel button to appear on custom statuses that only have the cancel checkbox checked
        add_filter( 'woocommerce_valid_order_statuses_for_cancel', array( &$this, 'cancel_button_handler' ) );
        add_filter( 'woocommerce_my_account_my_orders_actions', array( &$this, 'my_orders_actions' ), 10, 2 );


        add_filter( 'woocommerce_reports_order_statuses', array( &$this, 'display_in_reports_handler' ) );

        add_filter('set-screen-option', array($this, 'set_table_option'), 10, 3);

    }

    function statuses_table_options() {
        if(!isset($_GET['tab']) || $_GET['tab'] == 'list_status'){            
            $option = 'per_page';
            $args = array(
                'label' => __( 'Statuses', 'woocommerce_status_actions' ),
                'default' => 10,
                'option' => 'wc_custom_statuses_per_page',
            );
            add_screen_option( $option, $args );
            global $sa_table;
            $sa_table = new WC_Custom_Status_Table();
        }        
    }
    

 
    function set_table_option($status, $option, $value) {
     
        if ( 'wc_custom_statuses_per_page' == $option ) return $value;
     
        return $status;
     
    }
    function after_woocommerce_init(){
        add_action('admin_menu',array(&$this,'add_admin_pages'));
    }

    /**
     * Allow cancel button to appear beside orders that have custom status if administrator chose that.
     * @param  array $a the statuses names that allow the cancellation of the order
     * @return array    modified array (or original if not modified)
     */
    function cancel_button_handler( $a ) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'woocommerce_order_status_action';
        /* -- Preparing your query -- */
        $query = "SELECT * FROM {$table_name}";
        $records = $wpdb->get_results( $query );

        foreach( $records as $r ) {
            if( !empty( $r->can_cancel ) && $r->can_cancel == '1' ) {
                $a[] = $r->status_slug;
            }
        }
        return $a;
    }
    function my_orders_actions( $a, $order ) {
        
        global $wpdb;
        $table_name = $wpdb->prefix . 'woocommerce_order_status_action';
        /* -- Preparing your query -- */
        $query = "SELECT * FROM {$table_name}";
        $records = $wpdb->get_results( $query );

        foreach( $records as $r ) {
            $st_list  = unserialize($r->statuses_trigger);
            $order_st = $order->post_status;
            
            if( !empty( $r->trigger_status ) && $r->trigger_status == 1 && !empty($st_list) && in_array($order_st, $st_list) ) {
                $custom_action = 
                $a['mark_custom_status_'.$r->status_slug] = array(
                    'url' => wp_nonce_url( admin_url( "admin-ajax.php?action=woocommerce_mark_order_as_custom_status&order_id={$order->id}&custom_action_name={$r->status_slug}" ), "woocommerce-mark-order-{$r->status_slug}" ),
                    'name' => !empty($r->trigger_button_label) ? $r->trigger_button_label : $r->status_name,                    
                    );
            }
        }
        return $a;
    }

    public function display_in_reports_handler($statuses){
        if(!is_array($statuses))
            return $statuses;
        
        if(in_array('refunded', $statuses) && sizeof($statuses) == 1)
            return $statuses;
        global $wpdb;
        $table_name = $wpdb->prefix . 'woocommerce_order_status_action';
        /* -- Preparing your query -- */
        $query = "SELECT new_status FROM {$table_name} WHERE display_in_reports = '1'";
        $records = $wpdb->get_results( $query );
        $a = array();
        foreach( $records as $r ) {
            if( !empty( $r->new_status ) ) {
                $a[] = $r->new_status;
            }
        }
                
        $b = array_merge($statuses, $a);

        return $b;
    }

    /**
     * Delete status(es) - handles the ajax request
     * @return void
     */
    function woocommerce_status_actions_delete() {
        global $woocommerce;
        // If our required post parameters are set
        if( isset( $_POST['delete_status_id'] ) || isset( $_POST['status_id'] ) ) {
            // If not an administrator disallow it
            if ( !is_admin() ) { echo '0'; die(); }
            // If user can not edit shop orders disallow it
            if ( !current_user_can('edit_shop_orders') ) { echo '0'; die(); }
            // If nonce is not set or wrong, disallow it
            if( empty( $_POST['nonce'] ) || !wp_verify_nonce( $_POST['nonce'], 'delete_custom_status' ) ) { echo '0'; die(); }
            // If we're deleting a single status and the alternative term is set
            if( ! empty( $_POST['delete_status_id'] ) && ! empty( $_POST['alt_term'] ) ) { // single deletion

                // We're expecting ids - check for valid numbers
                if( is_numeric( $_POST['delete_status_id'] ) && (is_numeric( $_POST['alt_term'] ) || strpos($_POST['alt_term'], 'wc-') !== false ) ) {
                    // If the status id to delete is the same as the alternative term, disallow it
                    if( $_POST['delete_status_id'] == $_POST['alt_term'] ) {
                        echo '-1';
                        die();
                    }
                    // Ok Do it - delete the status
                    $sa = new WC_Custom_Status();
                    $sa->term_id = $_POST['delete_status_id'];
                    $deleted = $sa->delete();
                    if( !$deleted ) {
                        echo '0';
                    } else {
                        // Success
                        echo '1';
                    }
                    die();
                }
            } else if( !empty( $_POST['status_id'] ) ) {
                // We get a serialized form submission, parse it into params
                $params = array();
                parse_str( $_POST['status_id'], $params );
                // The array is status_id_to_delete => alternative_term_or_status_to_set_the_orders_to
                foreach( $params['status_id'] as $status_id => $alt_term ) {
                    // Ok Do it for every submitted term for deletion
                    $sa = new WC_Custom_Status();
                    $sa->term_id = $status_id;
                    $deleted = $sa->delete( $alt_term );
                    if( !$deleted ) {
                        echo '0';
                        die();
                    }
                }
                echo '1'; // if we reached here, everything was deleted successfully!
                die();
            }
            echo '0';
            die();
        }
    }

    /**
     * Adds our admin pages and the status and actions menu to the menu
     */
    function add_admin_pages() {
        //Options-writing.php is given as parent for pages that we do not want to display in the menu
        add_submenu_page('options-writing.php', __('Add New Status', 'woocommerce_status_actions'),  __('Add Status', 'woocommerce_status_actions') , 'manage_woocommerce', 'woocommerce_status_and_actions&tab=add_status', array(&$this,'create_settings_page'));

        add_submenu_page('options-writing.php', __('Custom Status', 'woocommerce_status_actions'),  __('Custom Status', 'woocommerce_status_actions') , 'manage_woocommerce', 'woocommerce_status_and_actions&tab=list_status', array(&$this,'create_settings_page'));

        add_submenu_page('options-writing.php', __('Edit Status', 'woocommerce_status_actions'),  __('Edit Status', 'woocommerce_status_actions') , 'manage_woocommerce', 'woocommerce_status_and_actions&tab=edit_status', array(&$this,'edit_status'));

        add_submenu_page('options-writing.php', __('Delete Status', 'woocommerce_status_actions'),  __('Delete Status', 'woocommerce_status_actions') , 'manage_woocommerce', 'woocommerce_status_and_actions&tab=delete_status', array(&$this,'delete_status'));
        
        $hook = add_submenu_page('woocommerce', __('Order Statuses', 'woocommerce_status_actions'),  __('Order Statuses', 'woocommerce_status_actions') , 'manage_woocommerce', 'woocommerce_status_and_actions', array(&$this,'create_settings_page'));

        add_action( "load-$hook", array($this, 'statuses_table_options') );
        

    }

    /**
     * Decides which page to render
     * @return void
     */
    public function create_settings_page() {
        global $current_tab, $current_section;
        global $sa_table;
        $tab = (isset($_GET['tab']) && !empty($_GET['tab'])) ? $_GET['tab'] : 'list_status';

        ?>
        <div class="wrap">
            
            <h2 class="nav-tab-wrapper woo-nav-tab-wrapper">
                <a class="nav-tab <?php echo $tab == 'list_status' || $tab == 'edit_status' || $tab == '' ? 'nav-tab-active' : ''; ?>" href="<?php echo admin_url(); ?>admin.php?page=woocommerce_status_and_actions">Edit Status</a>
                <a class="nav-tab <?php echo $tab == 'add_status' ? 'nav-tab-active' : ''; ?>" href="<?php echo admin_url(); ?>admin.php?page=woocommerce_status_and_actions&tab=add_status">Add Status</a>
                <a class="nav-tab <?php echo $tab == 'settings' ? 'nav-tab-active' : ''; ?>" href="<?php echo admin_url(); ?>admin.php?page=woocommerce_status_and_actions&tab=settings">Settings</a>
            </h2>
            <?php 
            $display = '';
            if( !empty( $_GET['success_add'] ) ) {
                $message = __('Order status created.', 'woocommerce_status_actions');
                $display = "<div id='message' class='updated below-h2'><p>{$message}</p></div>";
            }
            else if( !empty( $_GET['delete_add'] ) ) 
            {
                $message = __('Order status deleted.', 'woocommerce_status_actions');
                $display = "<div id='message' class='updated below-h2'><p>{$message}</p></div>";
            }
            else if( !empty( $_GET['updated'] ) ) 
            {
                $message = __('Order status updated.', 'woocommerce_status_actions');
                $display = "<div id='message' class='updated below-h2'><p>{$message}</p></div>";
            }
            echo $display;
             
            switch ($tab) {
                case 'edit_status':
                    $this->edit_status();
                    break;
                case 'add_status':
                    $this->add_status_page();
                    break;
                case 'settings':
                    $this->settings_page_init();
                    break;
                case 'delete_status':
                    $this->delete_status();
                    break;               
                default:
                    ?>
                    <form action="<?php echo admin_url();?>admin.php?page=woocommerce_status_and_actions&tab=delete_status" method="POST">
                    <?php
                        $actions_table = new WC_Custom_Status_Table();
                        $actions_table->prepare_items();
                        $actions_table->display(); 
                    ?>
                    </form>
                    <?php
                    break;
            }
            ?>
        </div>
        <?php

    }


    public function settings_page_init() {
        include_once( WC()->plugin_path() . '/includes/admin/settings/class-wc-settings-page.php' );
        include_once 'wc_custom_status_settings.php';

        $settings = new WC_Custom_Status_Settings();

        if( $settings->save() ): ?>
        <div id='message' class='updated below-h2'><p><?php _e( 'Settings saved.', 'woocommerce' ); ?></p></div>
        <?php endif; ?>
        <div class="wrap woocommerce">
            <form method="post" id="mainform" action="" enctype="multipart/form-data">
                
                <?php $settings->output(); ?>

                <p class="submit">
                    <?php if ( ! isset( $GLOBALS['hide_save_button'] ) ) : ?>
                        <input name="save" class="button-primary" type="submit" value="<?php _e( 'Save Changes', 'woocommerce' ); ?>" />
                    <?php endif; ?>
                    <?php wp_nonce_field( 'wc_custom_status_settings' ); ?>
                </p>
            </form>
        </div>
        <?php        
    }

    /**
     * Renders the delete page on bulk action and per single delete
     * @return [type] [description]
     */
    function delete_status() {
        // If status_id is not submitted view an error message
        if( ( empty( $_GET['status_id'] ) && empty( $_POST['status_id'] ) ) ) {
            ?>
            <div id='message' class='updated below-h2'>
                <p>
                    Something is wrong. Please go back and try again.
                </p>
            </div>
            <?php
        } else {
            // If the get parameter is set, it's a single deletion
            $arr = array();
            if( isset( $_GET['status_id'] ) ) {
                $arr[] = $_GET['status_id'];
                // Important for ajax handling - handled in custom-status-action-delete.js file
                $class = 'linen_confirm';
            } else {
                $arr = $_POST['status_id'];
                // Important for ajax handling
                $class = 'linen_confirm_multi';
            }
            $select_statuses = array();
            
            $order_statuses = wc_get_order_statuses();
            
            foreach ($arr as $st_id) {
                $status_slug = WC_Custom_Status::get_status_name_for_id($st_id);
                if( array_key_exists ('wc-'.$status_slug, $order_statuses) ){
                    $select_statuses[$status_slug] = $order_statuses['wc-'.$status_slug];
                    unset($order_statuses['wc-'.$status_slug]);
                }
            }
            
            $order_select = '';
            foreach( $order_statuses as $o => $o_name ) {
                $order_select .= '<option value="' . $o . '">' . $o_name . '</option>';
            }
           
            
            // Build the selectbox and the confirmation button
            
            ?>
            <div class="wrap">
                <h2>Delete Status</h2>
                <form action="POST" onsubmit="return false;">
            <?php
            foreach( $arr as $status_id ) {
                // Get the order status
                if(   defined( 'WC_VERSION') && floatval(WC_VERSION) > 2.1 ){
                    $status_slug = WC_Custom_Status::get_status_name_for_id($status_id);
                    
                    $status_name = $select_statuses[$status_slug];
                }else{
                    $t = get_term_by( 'id', $status_id, 'shop_order_status' );
                    $status_name = $t->name;
                }
                ?>
                <div id="wrapper_for_<?php echo $status_id; ?>">
                    <fieldset>
                    <ul style="list-style:none;">
                        <li>
                            <div id="select_for_<?php echo $status_id; ?>">
	                            <label for="delete_option1">Attribute all <?php echo $status_name; ?> orders to:</label>
                                <select name="status_id[<?php echo $status_id; ?>]" class="linen_order_statuses" data-id="<?php echo $status_id; ?>">
                                    <?php echo $order_select; ?>
                                </select>
                            </div>
                        </li>
                    </ul>
                    </fieldset>
                    </div>
                <?php
            }
        ?>
        <p class="submit"><a data-url="<?php echo admin_url('admin.php?page=woocommerce_status_and_actions&delete_add=true'); ?>" data-nonce="<?php echo wp_create_nonce('delete_custom_status'); ?>" class="button <?php echo $class; ?>" href="javascript:;">Confirm Deletion</a>
        </form>
        </div>
        <?php
        }
    }

    /**
     * Rendering the add status page
     */
    function add_status_page() {
        global $woocommerce;
             $display = '
        <div class="error" style="display: none;">
            <p></p>
        </div>';
        $sa = new WC_Custom_Status();
        //Logic for handling form submission
        if(isset($_POST['submit'])){
            try {
                // Add the status
                $sa = new WC_Custom_Status();
                $result = $sa->add_status($_POST);
                if($result) {
                    // Redirect on success
                    ?>
                    <script type="text/javascript">
                        window.location = '<?php echo admin_url('admin.php?page=woocommerce_status_and_actions&tab=list_status&success_add=true'); ?>';
                    </script>
                    <?php
                }
            } catch (RequiredFieldMissingException $e){
                $message = __('<h3>Required fields</h3> ','woocommerce_status_actions');
                $message .= '<ol>';
                foreach( $e->get_missing_fields() as $field ) {
                    $message .= '<li> The field <strong><i>' . WC_Custom_Status_Plugin::get_field_name($field) . '</i></strong> is required.</li>';
                }
                $message .= '</ol>';
                $display .= "<div class='error'>{$message}</div>";
            } catch (WPErrorException $e){
                $message = implode(',',$e->get_error_messages());
                $display .= "<div class='error'>{$message}</div>";
            } catch (Exception $e){
                $message = $e->getMessage();
                $display .= "<div class='error'>{$message}</div>";
            }
        }

        $email_to_html = '';
        $email_to = array('customer' => 'Customer','admin' => 'Administrator','both'=>'Administrator & Customer', 'custom' => 'Custom Email Address');
        foreach($email_to as $email_key => $email_val){
            if($email_key == $sa->email_to){
                $email_to_html .= "<option value='{$email_key}' selected='selected'>{$email_val}</option>";
            } else {
                $email_to_html .= "<option value='{$email_key}'>{$email_val}</option>";
            }
        }

        $status_color_type_html = '';
        $status_color_types = array(
                '1' => '<span class="old_status_icon status_edit woocommerce_status_font_icon woocommerce_status_actions_e372"></span>',
                '0' => '<span  class="new_status_icon status_edit woocommerce_status_font_icon woocommerce_status_actions_e372"></span>',
                '2' => '<strong class="status_text status_edit">preview</strong>',
                '3' => '<strong class="status_line_text status_edit">preview</strong>'
                );
        foreach($status_color_types as $sc_key => $sc_val){
            $status_color_type_html .= "<li><label for='radio_$sc_key'><input type='radio' name='status_color_type' id='radio_$sc_key' value='$sc_key'> $sc_val</label></li>";
        }
        $hide_bulk_action         = '';
        $dashboard_widget         = '';
        $product_reviews          = '';
        $order_is_editable        = '';
        $sends_email              = '';
        $check_display_in_reports = '';
        $download_permitted       = '';
        $stock_status             = '';
        $trigger_status           = '';
        $check_can_cancel         = '';
        $trigger_button_label     = '';
        $check_display_completed  = '';
        $check_display_processing = '';
        $display_custom_info      = '';
        $display_essential_info   = '';
        //Logic for displaying form
        ob_start();
        include_once 'add_status_form_html.php';
        $display .= ob_get_contents();
        ob_end_clean();

        wp_enqueue_script(
            'validate_custom_status',
            plugin_dir_url( __FILE__ ) . '../js/validate_custom_status.js',
            array('jquery','jquery-validate'),
            '1.0',
            true
        );
        echo $display;
    }

    /**
     * Edit page HTML
     * @return void
     */
    function edit_status(){
        global $woocommerce;
        if(isset($_REQUEST['status_id'])){
            $display = '<div class="wrap">
            <div class="error" style="display: none;">
                <p></p>
            </div>';
            try{
                $status_slug = WC_Custom_Status::get_status_name_for_id($_REQUEST['status_id']);
                if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'save_edit'){
                    $sa = new WC_Custom_Status();
                    if($sa->add_status($_POST,true)){
                        // Display success message
                        $message = __('Order status updated.','woocommerce_status_actions');
                        $display .= "<div id='message' class='updated below-h2'><p>{$message}</p></div>";
                    }
                }
                // Get already saved data to render in the HTML
                
                if($status_slug){
                    $sa = new WC_Custom_Status($status_slug);

                    $display_custom_info = '';
                    $display_essential_info = '';
                    if($sa->sends_email == '1'){
                        $sends_email = 'checked="checked"';
                        if($sa->display_custom_info == '1'){
                            $display_custom_info = 'checked="checked"';
                        }
                        if($sa->display_essential_info == '1'){
                            $display_essential_info = 'checked="checked"';
                        }
                    } else {
                        $sends_email = '';
                    }

                    if($sa->hide_bulk_action == '1'){
                        $hide_bulk_action = 'checked="checked"';
                    } else {
                        $hide_bulk_action = '';
                    }
                    
                    if(wcos_get_meta($sa->id, 'product_reviews', true) == 'yes'){
                        $product_reviews = 'checked="checked"';
                    } else {
                        $product_reviews = '';
                    }
                    if(wcos_get_meta($sa->id, 'order_is_editable', true) == 'yes'){
                        $order_is_editable = 'checked="checked"';
                    } else {
                        $order_is_editable = '';
                    }
                    if(wcos_get_meta($sa->id, 'dashboard_widget', true) == 'yes'){
                        $dashboard_widget = 'checked="checked"';
                    } else {
                        $dashboard_widget = '';
                    }
                    $email_to_html = '';
                    $email_to = array('customer' => 'Customer','admin' => 'Administrator','both'=>'Administrator & Customer', 'custom' => 'Custom Email Address');
                    foreach($email_to as $email_key => $email_val){
                        if($email_key == $sa->email_to){
                            $email_to_html .= "<option value='{$email_key}' selected='selected'>{$email_val}</option>";
                        } else {
                            $email_to_html .= "<option value='{$email_key}'>{$email_val}</option>";
                        }
                    }


                    $status_color_type_html = '';
                    $status_color_types = array(
                            '1' => '<span style="background-color:'.$sa->status_color.';" class="old_status_icon status_edit woocommerce_status_font_icon woocommerce_status_actions_' . $sa->status_icon . '"></span>',
                            '0' => '<span style="border: 1px solid '. $sa->status_color .'; color:'. $sa->status_color .';" class="new_status_icon status_edit woocommerce_status_font_icon woocommerce_status_actions_' . $sa->status_icon . '"></span>',
                            '2' => '<strong class="status_text status_edit" style="background-color:'.$sa->status_color.';">'.strtolower($sa->status_label).'</strong>',
                            '3' => '<strong class="status_line_text status_edit" style="color:'.$sa->status_color.';">'.strtolower($sa->status_label).'</strong>'
                            );
                    foreach($status_color_types as $sc_key => $sc_val){
                        $checked = ($sc_key == $sa->status_color_type)?'checked':'';
                        $status_color_type_html .= "<li><label for='radio_$sc_key'><input type='radio' name='status_color_type' id='radio_$sc_key' value='$sc_key' $checked> $sc_val</label></li>";
                    }

                    $check_display_completed  = ((!empty( $sa->display_completed ) ) && $sa->display_completed == '1') ? 'checked':'';
                    $check_display_processing = ((!empty( $sa->display_processing ) ) && $sa->display_processing == '1') ? 'checked':'';
                    $check_can_cancel         = ((!empty( $sa->can_cancel ) ) && $sa->can_cancel == '1') ? 'checked':'';
                    $check_display_in_reports = ((!empty( $sa->display_in_reports ) ) && $sa->display_in_reports == '1') ? 'checked':'';
                    $download_permitted       = ((!empty( $sa->download_permitted ) ) && $sa->download_permitted == '1') ? 'checked':'';
                    $stock_status             = $sa->stock_status;

                    $trigger_status       = ( (!empty( $sa->trigger_status ) ) && $sa->trigger_status == 1) ? 'checked':'';
                    $statuses_trigger     = $sa->statuses_trigger;
                    $trigger_button_label = !empty($sa->trigger_button_label) ? $sa->trigger_button_label : $sa->status_name;

                    ob_start();
                    include_once 'add_status_form_html.php';
                    $display .= ob_get_contents();
                    ob_end_clean();

                    wp_enqueue_script(
                        'validate_custom_status',
                        plugin_dir_url( __FILE__ ) . '../js/validate_custom_status.js',
                        array('jquery','jquery-validate'),
                        '1.0',
                        true
                    );
                }
            } catch (RequiredFieldMissingException $e){
                $message = __('Required fields ','woocommerce_status_actions');
                $message .= implode(',',$e->get_missing_fields());
                $message .= __(' missing','woocommerce_status_actions');
                $display .= "<div class='error'>{$message}</div>";
            } catch (WPErrorException $e){
                $message = implode(',',$e->get_error_messages());
                $display .= "<div class='error'>{$message}</div>";
            } catch (Exception $e){
                $message = $e->getMessage();
                $display .= "<div class='error'>{$message}</div>";
            }
            echo $display;
        }
    }

   

    /**
     * Process the status changes
     * @param  [int] $order_id   [order id]
     * @param  [string] $old_status [old status]
     * @param  [string] $new_status [new status]
     * @return boolean
     */
    function process_order_status($order_id,$old_status,$new_status){
        $default_statuses = array('pending','failed','on-hold','processing','completed','refunded','cancelled');
        // Process the status if not one of the defaults
        if(!in_array($new_status, $default_statuses)){
            $sa = new WC_Custom_Status();
            try{
                $sa->process_status($order_id,$old_status,$new_status);
            } catch(Exception $e){

            }
        }
        return true;
    }


    /**
     * What data to view in the tables' columns
     * @param  [string] $column [column name]
     * @return void
     */
    function payment_column_manage($column){
        global $post, $woocommerce, $the_order;

        if ( empty( $the_order ) || $the_order->id != $post->ID )
            $the_order = new WC_Order( $post->ID );

        if( $column == 'order_status' ) {

            $status = new WC_Custom_Status($the_order->status);
            $status_slug = $status->status_slug;
            
            if( !empty( $status_slug ) ) { // it's a custom one
                // Style!
                $style = '';
                $class = '';
                if( empty( $status->status_color_type ) || $status->status_color_type == '0' ) {
                    $style = 'style="border: 1px solid '. $status->status_color .'; color:'. $status->status_color .';"';
                    $class = 'new_status_icon';
                } else if( $status->status_color_type == '1' ){
                    $style = 'style="background-color: '.$status->status_color.';"';
                    $class = 'old_status_icon';

                }
                if( empty( $status->status_color_type ) || $status->status_color_type == '0' || $status->status_color_type == '1' ) {
                    echo '<span '.$style.'  class="default-status-icon-selector woocommerce_status_font_icon woocommerce_status_actions_'.$status->status_icon.' tips hhh '.$class.'" data-tip="'.$status->status_name.'" alt="'.$status->status_name.'"></span></td>';
                } else if( $status->status_color_type == '2' ){
                    echo '<strong class="status_orders_page status_text status_text_'. $status->status_slug .' hhh" style="background-color:'.$status->status_color.';">'.mb_strtolower($status->status_label).'</strong>';
                } else {
	                echo '<strong class="status_orders_page status_line_text status_line_text_'. $status->status_slug .' hhh" style="color:'.$status->status_color.';">'.mb_strtolower($status->status_label).'</strong>';
                }
                WC_Custom_Status_Plugin::hide_marks();
                
            } else {
                // It's probably a default
                $style = get_option( 'woocommerce_status_actions_default_style' );
                if( !empty( $style ) ) {
                    $modifed_def_st    = get_option( 'wc_custom_status_edit_existing_status', '' );
                    $slug = 'wc-'.$the_order->status;
                    $name = isset($modifed_def_st[$slug]) && !empty($modifed_def_st[$slug]['label']) ? $modifed_def_st[$slug]['label'] : $the_order->status;
                    switch( $style ) {
                        case '1':
                            echo '<strong class="status_orders_page status_text status_text_'. $the_order->status .' hhh">'. $name .'</strong>';
                            WC_Custom_Status_Plugin::hide_marks();
                        break;
                        case '2':
                            echo '<strong class="status_orders_page status_line_text status_line_text_'. $the_order->status .' hhh">'. $name .'</strong>';
                            WC_Custom_Status_Plugin::hide_marks();
                        break;
                        case '0':
                        default:
                        break;
                    }
                }
            }
        } # end of order status
    }

    /**
     * Adds the action button of our custom status - per order
     * @param  [array] $actions   [actions for the order to be displayed]
     * @param  [object] $the_order [order object]
     * @return [array]            the actions array with our buttons added
     */
    function action_column_manage($actions, $the_order){
        global $woocommerce;

        $custom_action_list = WC_Custom_Status::get_status_list_names();
        
        foreach($custom_action_list as $custom_action => $custom_action_name){
            $s = new WC_Custom_Status();
            $s->load_status_from_db( $custom_action );
            $display_for = $s->display_for;
            if(in_array($the_order->status, $display_for)){
                $status = new WC_Custom_Status($custom_action);

                ########## Version ##############

                $url = wp_nonce_url( admin_url( "admin-ajax.php?action=woocommerce_mark_order_as_custom_status&order_id=".$the_order->id."&custom_action_name=".$custom_action ), "woocommerce-mark-order-{$custom_action}" );
                if( defined('WC_VERSION') && version_compare( WC_VERSION, '2.3', '>=' ) )
                    $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=woocommerce_mark_order_status&status=' . $custom_action . '&order_id=' . $the_order->id ), 'woocommerce-mark-order-status' );
                    // Version 2.1
                    $actions[$custom_action] = array(
                        'url'       => $url,
                        'name'      => __( ucfirst($custom_action_name), 'woocommerce_status_actions' ),
                        'action'    => 'woocommerce_status_action_font_icon woocommerce_status_actions_' . $status->action_icon
                    );
            }
            if( $the_order->status == $custom_action ) {
                if( (!empty($s->display_completed)) && $s->display_completed == '1' ) {

                    $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=woocommerce_mark_order_complete&order_id=' . $the_order->id ), 'woocommerce-mark-order-complete' );
                    if( defined('WC_VERSION') && version_compare( WC_VERSION, '2.3', '>=' ) )
                        $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=woocommerce_mark_order_status&status=completed&order_id=' . $the_order->id ), 'woocommerce-mark-order-status' );
                    $actions['complete'] = array(
                        'url'       => $url,
                        'name'      => __( 'Complete', 'woocommerce' ),
                        'action'    => "complete"
                        );
                }
                if( (!empty($s->display_processing)) && $s->display_processing == '1' ) {

                    $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=woocommerce_mark_order_processing&order_id=' . $the_order->id ), 'woocommerce-mark-order-complete' );
                    if( defined('WC_VERSION') && version_compare( WC_VERSION, '2.3', '>=' ) )
                        $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=woocommerce_mark_order_status&status=processing&order_id=' . $the_order->id ), 'woocommerce-mark-order-status' );

                    $actions['processing'] = array(
                            'url'       => $url,
                            'name'      => __( 'Processing', 'woocommerce' ),
                            'action'    => "processing"
                        );
                }
            }
        }
        return $actions;
    }

    /**
     * Generates the thickbox content that contains all action icons for the user to choose from
     * @return [string] [HTML to display]
     */
    function generate_icons_list_from_img( $status = false ) {
        global $woocommerce;
        $display = '';
        $class_selector = ($status)?'status-icon-selector':'action-icon-selector';

            // Version 2.1 Icons
            $font_file = plugin_dir_path( dirname(__FILE__) ) . 'css/fonts/icomoon.svg';
            if( file_exists( $font_file ) ) {
                $file = file_get_contents( $font_file );
                $matches = array();
                preg_match_all( "/glyph unicode=\"(.*?)\"/", $file, $matches);
                foreach( $matches[1] as $unicode ) {
                    $class_name = str_replace('&#x', '', $unicode);
                    $class_name = str_replace(';', '', $class_name);
                    ob_start();
                    ?>
                    <span class="<?php echo $class_selector; ?>-21 woocommerce_status_action_font_icon woocommerce_status_actions_<?php echo $class_name; ?>" data-src="<?php echo $class_name; ?>" href="javascript:;"></span>
                    <?php
                    $display .= ob_get_clean();
                }
            }
        return $display;
    }


    /**
     * Checks that required fields exist
     * @param  [array] $required_fields [keys to look for]
     * @param  [array] $submit_method   [submitted data]
     * @return boolean
     */
    static function check_missing_fields($required_fields, $submit_method){

        $missing_fields = array();

        foreach($required_fields as $field_name){
            if(empty($submit_method[$field_name])){
                $missing_fields[] = $field_name;
            }
        }

        if(!empty($missing_fields)){
            throw new RequiredFieldMissingException($missing_fields);
        } else {
            return false;
        }
    }

    /**
     * Neat name to be displated in validation error messages
     * @param  [string] $slug [slug of the field]
     * @return [string]       [better name for the field]
     */
    static function get_field_name( $slug ) {
        switch ( $slug ) {
            case 'display_for':
                return 'action triggers';
            case 'status_name':
                return 'status name';
            default:
                return $slug;
        }
    }

    static function hide_marks() {
        ?>
        <script type="text/javascript">
            jQuery('.hhh').siblings('mark').hide();
        </script>
        <?php
    }

}
if (!function_exists('mb_strtolower')) {
    function mb_strtolower($text){
        return $text;
    }
}
?>