<?php
/**
 * Custom Status Settings
 *
 * @since     1.5.3
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

if ( ! class_exists( 'WC_Custom_Status_Settings' ) ) :

/**
 * WC_Custom_Status_Settings
 */
class WC_Custom_Status_Settings extends WC_Settings_Page {

    /**
     * Constructor
     */
    public function __construct() {
    }
    /**
     * Get settings array
     *
     * @return array
     */
    public function get_settings() {
        $settings[] = array(  'title' => __( 'General Settings', 'woocommerce_status_actions' ), 'type' => 'title', 'id' => 'custom_status_general_settings' );
        $settings[] = array(
                'title'    => __( 'Status Style', 'woocommerce_status_actions' ),
                'id'       => 'woocommerce_status_actions_default_style',
                'default'  => '0',
                'desc_tip'  =>  __( 'This will effect the default WooCommerce\'s statuses (e.g. Processing, Completed, etc.).', 'woocommerce' ),
                'type'     => 'select',
                'class'    => 'chosen_select',
                'css'      => 'min-width: 350px;',
                'options'  => array(
                    '0'      => __( 'Icon', 'woocommerce_status_actions' ),
                    '1' => __( 'Fill Text', 'woocommerce_status_actions' ),
                    '2' => __( 'Line Text', 'woocommerce_status_actions' )
                )
            );

            $status_list = wc_get_order_statuses();
            $settings[] = array(
                'title'    => __( 'Successful Payment', 'woocommerce_status_actions' ),
                'id'       => 'wc_custom_status_payment_complete_status',
                'default'  => 'wc-processing',
                'desc_tip'  =>  __( 'Choose what default status to have upon successful payment. Default: Processing.', 'woocommerce' ),
                'type'     => 'select',
                'class'    => 'chosen_select',
                'css'      => 'min-width: 350px;',
                'options'  => $status_list
            );
            

        $custom_settings = apply_filters( 'custom_status_general_settings', array() );

        if(!empty($custom_settings))
            $settings = $settings+$custom_settings;

        $settings[] = array( 'type' => 'sectionend', 'id' => 'custom_status_general_settings');

        $settings[] = array(  'title' => __( 'Core Order Statuses', 'woocommerce_status_actions' ), 'type' => 'title', 'id' => 'edit_default_status' );
        $settings['edit_existing_status'] = array(
                'id'       => 'wc_custom_status_edit_existing_status',
                'type'     => 'edit_existing_status',
            );
        $settings[] = array( 'type' => 'sectionend', 'id' => 'edit_default_status');

        return apply_filters( 'wc_custom_status_get_settings', $settings );
    }

    /**
     * Output the settings
     */
    public function output() {
        global $current_section;
        add_action( 'woocommerce_admin_field_edit_existing_status', array($this, 'edit_existing_status') );

        $settings = self::get_settings();

        WC_Admin_Settings::output_fields( $settings );
    }

    public function edit_existing_status($val)
    {
        $data = get_option( $val['id'], '' );
        remove_filter('wc_order_statuses', array(WC_SA(), 'woocommerce_status_actions_wc_order_statuses'), 10, 1);
        $order_statuses  = wc_get_order_statuses();
        

        $color_statuses  = WC_SA()->color_statuses;
        $default_editing = WC_SA()->default_editing;
        if( empty($data) ){
            $data = array();
            foreach ($order_statuses as $key => $value){
                if(isset($default_editing[$key]))
                    $data[$key]['item_editing'] = $default_editing[$key];
            }
        }
        ?>
        
        <table class="wp-list-table widefat fixed posts default_status_settings">
            <thead>
                <tr>
                    <th style="width: 70px; text-align: center;"><?php _e( 'Icon', 'woocommerce_status_actions' ); ?></th>
                    <th><?php _e( 'Slug', 'woocommerce_status_actions' ); ?></th>
                    <th><?php _e( 'Name', 'woocommerce_status_actions' ); ?></th>
                    <th><?php _e( 'Label', 'woocommerce_status_actions' ); ?></th>
                    <th class="status_colour"><?php _e( 'Colour', 'woocommerce_status_actions' ); ?></th>
                    <th style="text-align: center;"><?php _e( 'Bulk Actions', 'woocommerce_status_actions' ); ?> <img width="16" height="16" src="<?php echo WC()->plugin_url(); ?>/assets/images/help.png" class="help_tip" data-tip="<?php _e( 'Check this box to hide this status from the Bulk Actions menu.', 'woocommerce_status_actions' ); ?>"></th>
                    <th style="text-align: center;"><?php _e( 'Item Editing', 'woocommerce_status_actions' ); ?> <img width="16" height="16" src="<?php echo WC()->plugin_url(); ?>/assets/images/help.png" class="help_tip" data-tip="<?php _e( 'Check this box to enable item editing for this status.', 'woocommerce_status_actions' ); ?>"></th>
                </tr>
            </thead>
          <tbody>
          
            <?php foreach ($order_statuses as $key => $value) {
                $label = 'wc-' === substr( $key, 0, 3 ) ? substr( $key, 3 ) : $key
                ?>
            <tr valign="top" data-statuskey="<?php echo $key; ?>" class="wc_order_statuses">
                <th class="order_status column-order_status" scope="row">
                    <mark class="<?php echo $label; ?> tips" data-tip="<?php echo $value; ?>"><?php echo $value; ?></mark>
                </th>
                <th class="titledesc" scope="row">
                  <label><?php echo $key; ?></label>
                </th>
                <td class="forminp status_name">
                   <input type="text" name="<?php echo $val['id']; ?>[<?php echo $key; ?>][name]" value="<?php echo isset($data[$key]) && !empty($data[$key]['name']) ? $data[$key]['name'] : $value; ?>" class="statusname">
                </td>
                <td class="forminp status_label">
                   <input type="text" name="<?php echo $val['id']; ?>[<?php echo $key; ?>][label]" value="<?php echo isset($data[$key]) && !empty($data[$key]['label']) ? $data[$key]['label'] : $label; ?>"  class="statuslabel">
                </td>
                <td class="forminp">
                    <?php
                    $color = '';
                    if(isset($color_statuses[$key])){
                        $color = $color_statuses[$key];                        
                    }
                    if(isset($data[$key]) && !empty( $data[$key]['color']) )
                        $color = $data[$key]['color'];
                    ?>
                   <input type="text"
                    autocomplete="off"
                    class="change_colour statuscolor" 
                    name="<?php echo $val['id']; ?>[<?php echo $key; ?>][color]" 
                    value="<?php echo $color; ?>">
                </td>
                <td style="text-align: center; ">
                    <input type="checkbox" name="<?php echo $val['id']; ?>[<?php echo $key; ?>][hide_bulk]" value="yes" <?php echo isset($data[$key]) && isset($data[$key]['hide_bulk']) ? 'checked="checked"' : ''; ?>>
                </td>
                <td style="text-align: center; ">
                    <input class="default_editing" type="checkbox" name="<?php echo $val['id']; ?>[<?php echo $key; ?>][item_editing]" value="yes" <?php echo isset($data[$key]) && isset($data[$key]['item_editing']) ? 'checked="checked"' : ''; ?>>
                </td>
            </tr>
            <?php }?>
          </tbody>
          <tfoot>
              <tr>
                  <th colspan="7">
                      <input type="button" value="Reset Defaults" class="button-primary alignright" id="wc_order_statuses_reset_defaults">
                  </th>
              </tr>
          </tfoot>
        </table>
        <?php
    }

    
    /**
     * Save settings
     */
    public function save() {
        global $current_section;
        $settings = self::get_settings();

        unset($settings['edit_existing_status']);

        if( isset($_POST['wc_custom_status_edit_existing_status'])){
            $e_st = is_array($_POST['wc_custom_status_edit_existing_status']) ? $_POST['wc_custom_status_edit_existing_status'] : array();
            update_option('wc_custom_status_edit_existing_status', $e_st);
        }

        return WC_Admin_Settings::save_fields( $settings );
    }
}

endif;
