<?php
/**
 * Plugin Name: WooCommerce Order Status & Actions Manager
 * Plugin URI: http://codecanyon.net/item/woocommerce-customer-relationship-manager/6392174&ref=actualityextensions
 * Description: Allows users to manage WooCommerce order statuses, create the action button that triggers the status and set up what happens to the order when the status is applied.
 * Version: 1.7.2
 * Author: Actuality Extensions
 * Author URI: http://actualityextensions.com/
 * Tested up to: 4.2
 *
 * Copyright: (c) 2015 Actuality Extensions (info@actualityextensions.com)
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 *
 * @package     WC-Custom-Status
 * @author      Actuality Extensions
 * @category    Plugin
 * @copyright   Copyright (c) 2015, Actuality Extensions
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

if (!defined('ABSPATH'))
    exit; // Exit if accessed directly

if (function_exists('is_multisite') && is_multisite()) {
    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    if ( !is_plugin_active( 'woocommerce/woocommerce.php' ) )
        return;
}else{
    if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins'))))
        return; // Check if WooCommerce is active    
}


if (!class_exists('WoocommerceStatusActions')) {

/**
 * Main WoocommerceStatusActions Class
 *
 * @class WoocommerceStatusActions
 * @since 1.4.9
 */
class WoocommerceStatusActions {
     /**
     * @var string
     */
    public $version = '1.7.4';

    /**
     * @var string
     */
    public $db_version = '1.7.7';

    /**
     * @var WoocommerceStatusActions The single instance of the class
     * @since 1.4.9
     */
    protected static $_instance = null;

    /**
     * Main WoocommerceStatusActions Instance
     *
     * Ensures only one instance of WoocommerceStatusActions is loaded or can be loaded.
     *
     * @since 1.4.9
     * @static
     * @see WC_SA()
     * @return WoocommerceStatusActions - Main instance
     */
    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    /**
     * Cloning is forbidden.
     *
     * @since 1.4.9
     */
    public function __clone() {
        _doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'woocommerce' ), '1.9' );
    }

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.4.9
     */
    public function __wakeup() {
        _doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'woocommerce' ), '1.9' );
    }

    /**
     * WoocommerceStatusActions Constructor.
     * @access public
     * @return WoocommerceStatusActions
     */

    public function __construct() {
        ini_set('zlib.output_compression','off');

        $this->color_statuses = array(
            'wc-pending'    => '#ffba00',
            'wc-processing' => '#73a724',
            'wc-on-hold'    => '#999999',
            'wc-completed'  => '#2ea2cc',
            'wc-cancelled'  => '#a00a00',
            'wc-refunded'   => '#999999',
            'wc-failed'     => '#d0c21f',
            );

        $this->default_editing = array(
            'wc-pending'    => 'yes',
            'wc-on-hold'    => 'yes'
            );

        register_activation_hook( __FILE__, array($this, 'activate') );

        add_action( 'run_automatic_trigger', 'woocommerce_status_actions_run_automatic_trigger', 88, 3 );

        $this->includes();
        $this->check_version();

        if (function_exists('is_multisite') && is_multisite()) {
            add_action( 'wpmu_new_blog', array($this, 'new_blog'), 10, 6); 
        }

        add_filter('woocommerce_payment_complete_order_status', array($this, 'payment_complete_order_status'));

        add_action('plugins_loaded', array($this, 'wc_custom_status_init'), 0);


        add_action('admin_init', array($this, 'add_admin_stylesheets'));
        add_action( 'admin_footer', array($this, 'woocommerce_status_actions_bulk'), 11 );
        add_action( 'load-edit.php', array($this, 'woocommerce_status_actions_bulk_handler') );
        add_action( 'wc_order_is_editable', array($this, 'order_is_editable'), 50, 2 );


        add_filter('woocommerce_order_is_download_permitted', array($this, 'woocommerce_status_actions_order_is_download_permitted'), 10, 2);
        add_action('plugins_loaded', array($this, 'woocommerce_status_actions_order_load_plugin_textdomain') );

        add_filter('wc_order_statuses', array($this, 'woocommerce_status_actions_wc_order_statuses'), 10, 1);
        add_action( 'init', array($this, 'register_post_status') );

        add_action( 'admin_footer', array($this, 'edit_existing_status') );

        add_action( 'current_screen', array( $this, 'conditonal_includes' ) );

        add_action('admin_head', array($this, 'message_tc_button'));
        
		add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array($this, 'plugin_action_links' ));
		add_filter( 'plugin_row_meta', array($this, 'plugin_row_meta' ), 10, 2 );

        $this->allow_product_reviews = '';
        $this->get_allows_product_reviews();

    }

	/**
	 * Show action links on the plugin screen.
	 *
	 * @param	mixed $links Plugin Action links
	 * @return	array
	 */
	public function plugin_action_links( $links ) {
		$plugin_links = array();
    
	    $plugin_links['settings'] = sprintf( '<a href="%s" title="%s">%s</a>', '' . admin_url( 'admin.php?page=woocommerce_status_and_actions' ) . '', __( 'View Settings', 'woocommerce_status_actions' ), __( 'Settings', 'woocommerce_status_actions' ) );

	    return array_merge( $plugin_links, $links );
	}
	
	/**
	* Show row meta on the plugin screen.
	*
	* @param mixed $links Plugin Row Meta
	* @param mixed $file  Plugin Base file
	* @return  array
	*/
	public static function plugin_row_meta( $links, $file ) {
	if ( $file == plugin_basename(__FILE__) ) {
	  $row_meta = array(
	    'docs'    => '<a href="' . esc_url( apply_filters( 'wc_csa_docs_url', 'http://actualityextensions.com/documentation/woocommerce-order-status-actions-manager/' ) ) . '" title="' . esc_attr( __( 'View Documentation', 'woocommerce_status_actions' ) ) . '">' . __( 'Docs', 'woocommerce_status_actions' ) . '</a>',
	    'support' => '<a href="' . esc_url( apply_filters( 'wc_csa_docs_url', 'http://actualityextensions.com/contact/' ) ) . '" title="' . esc_attr( __( 'Visit Support', 'wc_crm' ) ) . '">' . __( 'Support', 'woocommerce_status_actions' ) . '</a>',
	  );
	
	  return array_merge( $links, $row_meta );
	}
	
	return (array) $links;
	}
	
	
	
    public function message_tc_button($value='')
    {
        global $typenow;
        // check user permissions
        if ( !current_user_can('manage_woocommerce') ) {
            return;
        }

        if( ! isset($_GET['page']) || $_GET['page'] != 'woocommerce_status_and_actions' || !isset($_GET['tab']) || ($_GET['tab'] != 'add_status' && $_GET['tab'] != 'edit_status') )
            return;
        // check if WYSIWYG is enabled
        if ( get_user_option('rich_editing') == 'true') {
            add_filter("mce_external_plugins", array( $this, "message_add_tinymce_plugin") );
            add_filter('mce_buttons', array( $this, 'message_register_tc_button') );
        }
    }

    public function message_add_tinymce_plugin($plugin_array) {
        $plugin_array['wc_cosa_tc_button'] = plugins_url( '/js/tc-button.js', __FILE__ );
        return $plugin_array;
    }

    public function message_register_tc_button($buttons) {        
        array_push($buttons, "wc_cosa_tc_button");
        return $buttons;
    }

    function conditonal_includes()
    {
        $screen = get_current_screen();

        switch ( $screen->id ) {
            case 'dashboard' :
                include( 'classes/wc_custom_status_dashboard.php' );
                break;
        }
    }

    function order_is_editable($result, $order)
    {
        global $editable_statuses;
        $status = $order->get_status();
        if($editable_statuses === null ){
            $editable_statuses = get_editable_statuses();
        }
        $result = in_array( $status, $editable_statuses );
        return $result;
    }


    function get_allows_product_reviews(){
        global $wpdb;
        $st = $wpdb->get_results("SELECT status_slug FROM {$wpdb->prefix}woocommerce_order_status_action os LEFT JOIN $wpdb->wc_osmeta wc_osmeta ON( wc_osmeta.wc_os_id = os.id AND wc_osmeta.meta_key = 'product_reviews' ) WHERE wc_osmeta.meta_value = 'yes' ");

        $allows = array(); 
        foreach ($st as $key => $value) {
            $allows[] = 'wc-'.$value->status_slug;
        }
        if( !empty($allows) )
            $this->allow_product_reviews = "'" . implode("', '", $allows) . "'";

        add_filter('query', array($this, 'wc_customer_bought_product'), 50, 1);            
    }

    function wc_customer_bought_product($query){        
        $query = trim($query);
        $pos  = strpos($query, "SELECT COUNT( DISTINCT order_items.order_item_id )");
        $pos2 = strpos($query, "posts.post_status IN ( 'wc-completed', 'wc-processing' ) AND");
        if($pos === 0 && $pos2 === 369 && !empty($this->allow_product_reviews)){
            $query = str_replace("posts.post_status IN ( 'wc-completed', 'wc-processing' ) AND", "posts.post_status IN ( 'wc-completed', 'wc-processing', {$this->allow_product_reviews} ) AND", $query);
        }
        return $query;
    }

    function payment_complete_order_status($new_order_status){
        if($new_order_status == 'processing'){
            $complete_status = get_option('wc_custom_status_payment_complete_status');
            if($complete_status && !empty($complete_status))
                $new_order_status = substr($complete_status, 3);
        }
        return $new_order_status;
    }
    function check_version(){
        $installed_ver = get_option( "woocommerce_status_actions_db_version" );
        if( $installed_ver != $this->db_version ){
            $this->install();
        }
    }

    public function new_blog($blog_id, $user_id, $domain, $path, $site_id, $meta ) {
        global $wpdb;
        $pos_path = basename(dirname(__FILE__));
        if (is_plugin_active_for_network($pos_path.'/wc_custom_action_status.php')) {
            $old_blog = $wpdb->blogid;
            switch_to_blog($blog_id);
            $this->install();
            switch_to_blog($old_blog);
        }
    }
    public function activate($networkwide) {
        global $wpdb;
        if (function_exists('is_multisite') && is_multisite()) {
            // check if it is a network activation - if so, run the activation function for each blog id
            if ($networkwide) {
                $old_blog = $wpdb->blogid;
                // Get all blog ids
                $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");
                foreach ($blogids as $blog_id) {
                    switch_to_blog($blog_id);
                    $this->install();
                }
                switch_to_blog($old_blog);
                return;
            }   
        }          
        $this->install();
    }
    
    public function includes() {
        require dirname(__FILE__) . '/functions.php';
        require dirname(__FILE__) . '/classes/wc_custom_status_plugin.class.php';
        require dirname(__FILE__) . '/classes/wc_custom_status.class.php';
        require dirname(__FILE__) . '/classes/exceptions/requiredfieldmissing.class.php';
        require dirname(__FILE__) . '/classes/exceptions/wperrorexception.class.php';

        if(!class_exists('WP_List_Table')){
            require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
        }

        require dirname(__FILE__) . '/classes/tables/wc_custom_status_table.class.php';
    }

    function woocommerce_status_actions_order_load_plugin_textdomain() {
        load_plugin_textdomain( 'woocommerce_status_actions', false, dirname( plugin_basename( __FILE__ ) ) . '/lang/' );
    }

    

    function woocommerce_status_actions_wc_order_statuses($statuses)
    {
        global $wpdb;

        $changed_statuses = get_option( 'wc_custom_status_edit_existing_status', '' );
        
        foreach ($statuses as $key => $value) {
            if(isset($changed_statuses[$key]) && !empty($changed_statuses[$key]['name']))
                $statuses[$key] = $changed_statuses[$key]['name'];
        }

        $table_name = $wpdb->prefix . "woocommerce_order_status_action";
        $query = "SELECT * FROM $table_name";
        $status_info = $wpdb->get_results($query);

        foreach ($status_info as $status) {
            $statuses['wc-'.$status->status_slug] = $status->status_name;
        }
        return $statuses;
    }

    function register_post_status() {
        global $wpdb;
        $table_name = $wpdb->prefix . "woocommerce_order_status_action";
        $query = "SELECT * FROM $table_name";
        $status_info = $wpdb->get_results($query);

        foreach ($status_info as $status) {
            register_post_status( 'wc-'.$status->status_slug, array(
                'label'                     => $status->status_name,
                'public'                    => true,
                'exclude_from_search'       => false,
                'show_in_admin_all_list'    => true,
                'show_in_admin_status_list' => true,
                'label_count'               => _n_noop( $status->status_name.' <span class="count">(%s)</span>', $status->status_name.' <span class="count">(%s)</span>', 'woocommerce_status_actions' )
            ) );
        }

        
    }

    function woocommerce_action_status_compile_less( $less_file, $css_file ) {
        if ( ! class_exists( 'lessc' ) )
            include_once( WC()->plugin_path() . '/includes/libraries/class-lessc.php' );

        if ( ! class_exists( 'cssmin' ) )
            include_once( WC()->plugin_path() . '/includes/libraries/class-cssmin.php' );

        $less           = new lessc( $less_file );
        $compiled_css   = $less->parse();

        if ( $compiled_css )
            file_put_contents( $css_file, $compiled_css );
    }

    function woocommerce_status_actions_generate_action_icons() {
        ############ GENERATE CSS FILE ##################
        $font_file = plugin_dir_path( __FILE__ ) . 'css/fonts/icomoon.svg';
        if( file_exists( $font_file ) ) {
            $file = file_get_contents( $font_file );
            $matches = array();
            preg_match_all( "/glyph unicode=\"(.*?)\"/", $file, $matches);
            foreach( $matches[1] as $unicode ) {
                $class_name = str_replace('&#x', '', $unicode);
                $class_name = str_replace(';', '', $class_name);
                file_put_contents( plugin_dir_path( __FILE__ ) . '/css/action_icons.less' ,  '.woocommerce_status_actions_'. $class_name .':after {content: "\\'.$class_name.'";}' . PHP_EOL, FILE_APPEND);
            }
            // Compile Icons CSS
            $less_file = plugin_dir_path(__FILE__ ) . '/css/action_icons.less';
            $css_file = plugin_dir_path(__FILE__ ) . '/css/action_icons.css';
            woocommerce_action_status_compile_less( $less_file, $css_file );
        }
    }

    function woocommerce_custom_status_action_column_width_override() {
        ?>
        <script type="text/javascript">
            jQuery(document).ready(function(){
                if(!jQuery('table.resizablecolumns').length){
                    if( jQuery('.status_text.hhh').length > 0 ) {
                        jQuery('.column-order_status').css( 'width', '80px' );
                    }
                    else if ( jQuery('.status_line_text.hhh').length > 0 ) {
                        jQuery('.column-order_status').css( 'width', '80px' );
                    }
                }
            });
        </script>
        <?php
    }

    /**
     * Adds the plugin's stylesheets to the dashboard
     */
    function add_admin_stylesheets() {
        
        // Ensure our specific CSS is loaded only on our pages so as to not affect other pages
        if( (isset( $_GET['page'] ) && strpos( $_GET['page'], 'woocommerce_status_and_actions') != -1) || ( isset($_GET['post_type']) && strpos( $_GET['post_type'], 'shop_order') != -1) ) {
            add_action( 'admin_footer',  array($this, 'woocommerce_custom_status_action_column_width_override') );
        }


        if( !file_exists( plugin_dir_path(__FILE__ ) . '/css/action_icons.less' ) )
            woocommerce_status_actions_generate_action_icons();

        $action_icons_file_url = plugin_dir_url(__FILE__) . '/css/action_icons.css';


        if( !file_exists( plugin_dir_path(__FILE__ ) . '/css/admin.css' ) ) {
            // Compile Admin CSS
            $less_file = plugin_dir_path(__FILE__ ) . '/css/admin.less';
            $css_file = plugin_dir_path(__FILE__ ) . '/css/admin.css';
            woocommerce_action_status_compile_less( $less_file, $css_file );

        }

        $css_file_url = plugin_dir_url(__FILE__) . '/css/admin.css';

        wp_enqueue_style('status_actions_less', $css_file_url );
        wp_enqueue_style('status_actions_less_icons_test', $action_icons_file_url );

        wp_register_style( 'woocommerce_frontend_styles', plugins_url() . '/woocommerce/assets/css/admin.css' );
        wp_enqueue_style( 'woocommerce_frontend_styles' );


        wp_enqueue_script( 'woocommerce_tiptip_js', plugins_url() . '/woocommerce/assets/js/jquery-tiptip/jquery.tipTip.min.js' );


        wp_enqueue_style('wp-color-picker');
        wp_enqueue_script('wp-color-picker');

         if ( defined('WC_VERSION') && version_compare( WC_VERSION, '2.3', '>=' ) ) {
                if( !wp_script_is( 'select2', 'enqueued' ) ){
                    $suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';
                    wp_register_script( 'select2', WC()->plugin_url() . '/assets/js/select2/select2' . $suffix . '.js', array( 'jquery' ), '3.5.2' );
                    wp_register_script( 'wc-enhanced-select', WC()->plugin_url() . '/assets/js/admin/wc-enhanced-select' . $suffix . '.js', array( 'jquery', 'select2' ), WC_VERSION );
                    wp_localize_script( 'select2', 'wc_select_params', array(
                        'i18n_matches_1'            => _x( 'One result is available, press enter to select it.', 'enhanced select', 'woocommerce' ),
                        'i18n_matches_n'            => _x( '%qty% results are available, use up and down arrow keys to navigate.', 'enhanced select', 'woocommerce' ),
                        'i18n_no_matches'           => _x( 'No matches found', 'enhanced select', 'woocommerce' ),
                        'i18n_ajax_error'           => _x( 'Loading failed', 'enhanced select', 'woocommerce' ),
                        'i18n_input_too_short_1'    => _x( 'Please enter 1 or more characters', 'enhanced select', 'woocommerce' ),
                        'i18n_input_too_short_n'    => _x( 'Please enter %qty% or more characters', 'enhanced select', 'woocommerce' ),
                        'i18n_input_too_long_1'     => _x( 'Please delete 1 character', 'enhanced select', 'woocommerce' ),
                        'i18n_input_too_long_n'     => _x( 'Please delete %qty% characters', 'enhanced select', 'woocommerce' ),
                        'i18n_selection_too_long_1' => _x( 'You can only select 1 item', 'enhanced select', 'woocommerce' ),
                        'i18n_selection_too_long_n' => _x( 'You can only select %qty% items', 'enhanced select', 'woocommerce' ),
                        'i18n_load_more'            => _x( 'Loading more results&hellip;', 'enhanced select', 'woocommerce' ),
                        'i18n_searching'            => _x( 'Searching&hellip;', 'enhanced select', 'woocommerce' ),
                    ) );
                    wp_localize_script( 'wc-enhanced-select', 'wc_enhanced_select_params', array(
                        'ajax_url'                         => admin_url( 'admin-ajax.php' ),
                        'search_products_nonce'            => wp_create_nonce( 'search-products' ),
                        'search_customers_nonce'           => wp_create_nonce( 'search-customers' )
                    ) );

                    wp_enqueue_script('wc-enhanced-select');
                }

            }else{
                if( !wp_script_is( 'chosen', 'enqueued' ) ){
                    wp_register_script('chosen_js', WC()->plugin_url() . '/assets/js/chosen/chosen.jquery.min.js', array('jquery'), '2.66', true);
                    wp_register_script('ajax-chosen_js', WC()->plugin_url() . '/assets/js/chosen/ajax-chosen.jquery.min.js', array('jquery'), '2.66', true);

                    wp_enqueue_script('chosen_js');
                    wp_enqueue_script('ajax-chosen_js');
                }         
            }

        wp_enqueue_script( 'select2', WC()->plugin_url() . '/assets/js/select2/select2.min.js', array( 'jquery' ), '3.5.2' );

        wp_enqueue_style('woocommerce_frontend_styles', WC()->plugin_url() . '/assets/css/admin.css');


        wp_enqueue_script(
            'jquery-validate',
            plugin_dir_url( __FILE__ ) . 'js/jquery.validate.min.js',
            array('jquery'),
            '1.11.0',
            true
        );
        wp_enqueue_script('media-upload');
        wp_enqueue_script(
            'retina',
            plugin_dir_url( __FILE__ ) . 'js/retina.js',
            '0.0.2',
            true
        );
        
        // Handles the deletion of the statuses
        wp_enqueue_script( 'delete-custom-scripts', plugin_dir_url(__FILE__) . 'js/custom-status-action-delete.js', array('jquery') );

        /**************** iconpicker *******************/
        if(isset($_GET['page']) && $_GET['page'] == 'woocommerce_status_and_actions'){
            wp_register_style( 'iconpicker_styles', plugin_dir_url(__FILE__ ) . 'css/fontawesome-iconpicker.css' );
            wp_enqueue_style( 'iconpicker_styles' );

            wp_register_script( 'bootstrap_js', plugin_dir_url(__FILE__ ) .  'js/bootstrap.js', array('jquery') );
            wp_enqueue_script( 'bootstrap_js' );

            
            wp_register_script( 'iconpicker', plugin_dir_url(__FILE__ ) .  'js/fontawesome-iconpicker.js', array('jquery') );
            wp_enqueue_script( 'iconpicker' );

            wp_register_script( 'woocommerce_status_and_actions_scripts', plugin_dir_url(__FILE__ ) .  'js/js.js', array('jquery', 'bootstrap_js') );
            wp_enqueue_script( 'woocommerce_status_and_actions_scripts' );

            

            $display = array();
                // Version 2.1 Icons
                $font_file = plugin_dir_path( __FILE__ ) . 'css/fonts/icomoon.svg';
                if( file_exists( $font_file ) ) {
                    $file = file_get_contents( $font_file );
                    $matches = array();
                    preg_match_all( "/glyph unicode=\"(.*?)\"/", $file, $matches);
                    foreach( $matches[1] as $unicode ) {
                        $class_name = str_replace('&#x', '', $unicode);
                        $class_name = str_replace(';', '', $class_name);
                        if( $class_name == '20') continue;
                        $display[] = $class_name;
                    }
                }

            remove_filter('wc_order_statuses', array($this, 'woocommerce_status_actions_wc_order_statuses'), 10, 1);
            $order_statuses = wc_get_order_statuses();
            add_filter('wc_order_statuses', array($this, 'woocommerce_status_actions_wc_order_statuses'), 10, 1);


            $defaults = array(
                'labels'  => $order_statuses,
                'colors'  => $this->color_statuses,
                'editing' => $this->default_editing
                );

            $editor_btns = get_option('wc_fields_additional');

            $acf_btns    = wc_sa_get_acf_editor_btns();

            wp_localize_script( 'woocommerce_status_and_actions_scripts', 'wc_sa_icons_array', $display );
            wp_localize_script( 'woocommerce_status_and_actions_scripts', 'wc_sa_defaults', $defaults );
            wp_localize_script( 'woocommerce_status_and_actions_scripts', 'wc_sa_editor_btns', $editor_btns );
            wp_localize_script( 'woocommerce_status_and_actions_scripts', 'wc_sa_acf_editor_btns', $acf_btns );
        }
        /**************** /iconpicker *******************/
    }


    /**
     * Registers the hooks required for the plugin
     * @return void
     */
    function wc_custom_status_init() {
       
        // Email handling
        define( 'WC_CUSTOM_STATUS_PLUGIN_PATH', plugin_dir_url(__FILE__) );

        // Localization domain
        load_plugin_textdomain('woocommerce_status_actions', false, dirname( plugin_basename( __FILE__ ) ) . '/languages');

        // The core of the plugin
        $GLOBALS['wc_custom_status_object'] = new WC_Custom_Status_Plugin();

        // Adds the action hook for when the action button is clicked from the orders table (action column)
        $action_list = WC_Custom_Status::get_status_list();
        add_action("wp_ajax_woocommerce_mark_order_as_custom_status", array($this, 'process_custom_action') );
        add_action('admin_notices', array($this, 'wc_custom_order_bulk_admin_notices') );
    }


    /**
     * Adds the admin messages on successful changes/update of our custom actions/statuses
     * @return [type] [description]
     */
    function wc_custom_order_bulk_admin_notices() {
        global $post_type, $pagenow;

        if (isset($_REQUEST['order_status_changed'])) {
            $number = absint( $_REQUEST['order_status_changed'] );
            $message_title = __('Order status changed.','woocommerce_status_actions');
        } else {
            return;
        }

        if ( 'edit.php' == $pagenow && 'shop_order' == $post_type ) {
            $message = sprintf( __( $message_title . ' %s orders affected.', $number, 'woocommerce_status_actions' ), number_format_i18n( $number ) );
            echo '<div class="updated"><p>' . $message . '</p></div>';
        }
    }

    function woocommerce_status_actions_bulk() {
       global $post_type;
       $status_list_names = WC_Custom_Status::get_status_list_names();       
       $hide_statuses     = WC_Custom_Status::get_hide_statuse_for_bulk();       
       
       
       $need_remove       = array('processing', 'completed', 'on-hold');
       $order_statuses    = wc_get_order_statuses();
       $modifed_def_st    = get_option( 'wc_custom_status_edit_existing_status', '' );

       asort($order_statuses);

       if ( 'shop_order' == $post_type ) {
           ?>
           <script type="text/javascript">
               jQuery(document).ready(function($) {
                   $('.custom_payment_status').each(function(){jQuery(this).parent().parent().css('background-color',jQuery(this).attr('rowcolor'));});

                   <?php foreach($need_remove as $slug){ ?>
                        $('select[name="action"] option[value="mark_<?php echo $slug; ?>"], select[name="action2"] option[value="mark_<?php echo $slug; ?>"]').remove();
                    <?php } ?>

                    <?php foreach($order_statuses as $slug => $name){
                        if ($hide_statuses && in_array($slug, $hide_statuses)) continue;
                        if (isset($modifed_def_st[$slug]['hide_bulk'])) continue;

                        $label = 'wc-' === substr( $slug, 0, 3 ) ? substr( $slug, 3 ) : $slug;
                        if(isset($modifed_def_st[$slug]) && !empty($modifed_def_st[$slug]['label'])){
                            $label = $modifed_def_st[$slug]['label'];
                        }
                        else if(isset($status_list_names[$label]) ){
                            $label = $status_list_names[$label];
                        }
                        ?>
                        jQuery('<option>').val('mark_custom_status_<?php echo $slug; ?>')
                            .text('<?php _e( "Mark", "woocommerce_status_actions" ) ?> <?php echo $label; ?>').appendTo('select[name=action]');
                        jQuery('<option>').val('mark_custom_status_<?php echo $slug; ?>')
                            .text('<?php _e( "Mark", "woocommerce_status_actions" ) ?> <?php echo $label; ?>').appendTo('select[name=action2]');
                    <?php } ?>
                });
            </script>
            <?php
        }
    }

    function woocommerce_status_actions_bulk_handler() {
        if(!isset($_REQUEST['post'])){
            return;
        }
        $wp_list_table = _get_list_table( 'WP_Posts_List_Table' );
        $action = $wp_list_table->current_action();
        #print_r($action);

        $changed = 0;
        $post_ids = array_map( 'absint', (array) $_REQUEST['post'] );
        if(strstr($action,'mark_custom_status')) {
           $new_status = substr($action,strlen('mark_custom_status_'));

            if( strpos($new_status,'wc-') === 0 ){
                $new_status = substr($new_status,strlen('wc-'));                
            }
           $st = WC_Custom_Status::get_status_by_slug($new_status);

           $report_action = "order_status_changed";
           foreach( $post_ids as $post_id ) {
               $order = new WC_Order($post_id);
               $note = apply_filters('bulk_handler_custom_action_note', '', $new_status, $order );
               $order->update_status( $new_status, $note );
                if($st && $st->stock_status == 'reduce'){
                    $order->reduce_order_stock();
                }
                else if($st && $st->stock_status == 'restore'){
                    $this->restore_order_stock($order);
                }

               wc_delete_shop_order_transients( $post_id );
               $changed++;
           }
        } else{
            return;
        }
        $sendback = esc_url_raw(add_query_arg( array( 'post_type' => 'shop_order', $report_action => $changed, 'ids' => join( ',', $post_ids ) ), '' ) );
        wp_redirect( $sendback );
        exit();
    }

    public function restore_order_stock( $order ) {
            $order_items    = $order->get_items();

            if ( ! get_option('woocommerce_manage_stock') == 'yes' && ! empty( $order_items ) ) {
                return;
            }

            foreach ( $order_items as $item ) {

                if ( $item['product_id'] > 0 ) {
                    $_product = $order->get_product_from_item( $item );

                    if ( $_product && $_product->exists() && $_product->managing_stock() ) {

                        $old_stock = $_product->stock;

                        $qty = apply_filters( 'woocommerce_order_item_quantity', $item['qty'], $this, $item );

                        $new_quantity = $_product->increase_stock( $qty );

                        do_action( 'woocommerce_auto_stock_restored', $_product, $item );

                        $order->add_order_note( sprintf( __( 'Item #%s stock incremented from %s to %s.', 'woocommerce' ), $item['product_id'], $old_stock, $new_quantity) );

                        $order->send_stock_notifications( $_product, $new_quantity, $item['qty'] );
                    }
                }
            }
        } // End restore_order_stock()


    /**
     * Processes the custom action to mark an order with our custom status
     * @return void
     */
    function process_custom_action(){
            
        do_action('before_process_custom_action');

        if(isset($_REQUEST['custom_action_name']) && !empty($_REQUEST['custom_action_name']) ){

            $action_name = $_REQUEST['custom_action_name'];
            
            $order_id = isset($_REQUEST['order_id']) && (int) $_REQUEST['order_id'] ? (int) $_REQUEST['order_id'] : '';
            if (!$order_id) die;
            
            $wc_custom_action = new WC_Custom_Status();

            $test = $wc_custom_action->load_status_from_db($action_name);
            
            if ( !current_user_can('edit_shop_orders') ){
                if($wc_custom_action->trigger_status != '1')
                    wp_die( __( 'You do not have sufficient permissions to access this page.', 'woocommerce' ) );
            }

            if ( !check_admin_referer("woocommerce-mark-order-{$action_name}"))
                wp_die( __( 'You have taken too long. Please go back and retry.', 'woocommerce' ) );

            if( $test ){
                $order = new WC_Order( $order_id );
                if($order){
                    $note = apply_filters('process_custom_action_note', '', $action_name, $order );
                    $order->update_status( $action_name, $note );
                }
            }
            
        }

        do_action('after_process_custom_action');

        wp_safe_redirect( wp_get_referer() );

        die();
    }

    /**
     * On plugin's installation/activation, create our custom database tables
     * @return void
     */
    function install(){
        global $wpdb;
          $wpdb->hide_errors();
          $installed_ver = get_option( "woocommerce_status_actions_db_version" );
            $collate = '';

            if ( $wpdb->has_cap( 'collation' ) ) {
              if ( ! empty($wpdb->charset ) ) {
                $collate .= "DEFAULT CHARACTER SET $wpdb->charset";
              }
              if ( ! empty($wpdb->collate ) ) {
                $collate .= " COLLATE $wpdb->collate";
              }
            }
            $wpdb->show_errors();
            $table_name = $wpdb->prefix . "woocommerce_order_status_action";
            $sql = "CREATE TABLE $table_name (
                    id INT NOT NULL AUTO_INCREMENT,
                    created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
                    updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
                    status_slug VARCHAR(17) NOT NULL,
                    status_name VARCHAR(255) NOT NULL,
                    status_label VARCHAR(55) NOT NULL,
                    stock_status VARCHAR(55) NOT NULL,
                    status_icon TEXT NOT NULL,
                    status_color VARCHAR(10) NOT NULL,
                    status_color_type VARCHAR(1) NOT NULL,
                    sends_email VARCHAR(1) NOT NULL,
                    display_completed VARCHAR(1) NOT NULL,
                    display_processing VARCHAR(1) NOT NULL,
                    can_cancel VARCHAR(1) NOT NULL,
                    display_in_reports VARCHAR(1) NOT NULL,
                    download_permitted VARCHAR(1) NOT NULL,
                    email_to VARCHAR(15),
                    custom_email_address VARCHAR(100),
                    email_subject VARCHAR(200),
                    from_name VARCHAR(200),
                    from_address VARCHAR(100),
                    display_essential_info VARCHAR(1) NOT NULL,
                    display_custom_info VARCHAR(1) NOT NULL,
                    custom_title VARCHAR(200),
                    custom_info TEXT,
                    payment_status VARCHAR(20),
                    action_icon TEXT NOT NULL,
                    display_for TEXT NOT NULL,
                    new_status VARCHAR(55) NOT NULL,
                    trigger_status boolean DEFAULT false,
                    statuses_trigger TEXT NOT NULL,
                    trigger_button_label VARCHAR(150) NOT NULL,
                    hide_bulk_action VARCHAR(1) NOT NULL,
                    PRIMARY KEY  (id)
                   ) " . $collate;

            error_log($sql);
            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
            dbDelta( $sql );

            $table_name = $wpdb->prefix . "wc_osmeta";
            $sql = "CREATE TABLE $table_name (
                    meta_id bigint(20) unsigned NOT NULL auto_increment,
                    wc_os_id bigint(20) unsigned NOT NULL default '0',
                    meta_key varchar(255) default NULL,
                    meta_value longtext,
                    PRIMARY KEY  (meta_id),
                    KEY post_id (wc_os_id),
                    KEY meta_key (meta_key)
                ) " . $collate;
            dbDelta( $sql );

            if( version_compare( $installed_ver, '1.7.6', '<=' ) ){
                $data   = get_option( 'wc_custom_status_edit_existing_status' );
                if(!empty($data) && is_array($data)){
                    $default_editing = $this->default_editing;
                    foreach ($default_editing as $st => $a) {
                        $data[$st]['item_editing'] = 'yes';
                    }
                    update_option( 'wc_custom_status_edit_existing_status', $data );
                }
            }

            if(get_option( "woocommerce_status_actions_db_version" )) {
              update_option( "woocommerce_status_actions_db_version", $this->db_version );
            }else{
              add_option( "woocommerce_status_actions_db_version", $this->db_version );
            }
    }

    function wc_sa_get_woo_version() {
            // If get_plugins() isn't available, require it
        if ( ! function_exists( 'get_plugins' ) )
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        
            // Create the plugins folder and file variables
        $plugin_folder = get_plugins( '/' . 'woocommerce' );
        $plugin_file = 'woocommerce.php';
        
        // If the plugin version number is set, return it 
        if ( isset( $plugin_folder[$plugin_file]['Version'] ) ) {
            return $plugin_folder[$plugin_file]['Version'];

        } else {
        // Otherwise return null
            return NULL;
        }
    }

    function woocommerce_status_actions_order_is_download_permitted($download_permitted, $class)
    {
        $post_status = $class->post_status;
        $status_slug = str_replace ('wc-', '', $post_status);
        global $wpdb;
        $table_name = $wpdb->prefix . "woocommerce_order_status_action";

        $query = "SELECT * FROM $table_name WHERE status_slug = '$status_slug'";
        $status_info = $wpdb->get_row($query);
        if($status_info && !empty($status_info) && is_string($status_info->download_permitted) & $status_info->download_permitted == '1')
            return true;

        return $download_permitted;
    }

    function edit_existing_status(){
        $status_list_names = WC_Custom_Status::get_status_list_names();
        

        $order_statuses    = wc_get_order_statuses();
        $modifed_def_st    = get_option( 'wc_custom_status_edit_existing_status');

        $color_statuses = array(
            'wc-pending'    => '#ffba00',
            'wc-processing' => '#73a724',
            'wc-on-hold'    => '#999999',
            'wc-completed'  => '#2ea2cc',
            'wc-cancelled'  => '#a00a00',
            'wc-refunded'   => '#999999',
            'wc-failed'     => '#d0c21f',
            );
        ?>
        <style>
            <?php
            foreach ($order_statuses as $key => $value) {
                $slug  = 'wc-' === substr( $key, 0, 3 ) ? substr( $key, 3 ) : $key;
                $color = '';
                $order_status = false;
                
                if( isset($modifed_def_st[$key]) && !empty($modifed_def_st[$key]['color']) ){
                    $color = $modifed_def_st[$key]['color'];
                }elseif( isset($color_statuses[$key]) ){
                    $color = $color_statuses[$key];
                }else if(isset($status_list_names[$slug])){
                    $order_status = wc_get_custom_status($slug);
                    $color        = $order_status->status_color;
                }
                ?>
                    .post-type-shop_order .status_line_text_<?php echo $slug; ?>,
                    .widefat .column-order_status mark.<?php echo $slug; ?>:after{
                        color: <?php echo $color; ?>;
                    }
                    .post-type-shop_order .status_text_<?php echo $slug; ?>{
                        background-color: <?php echo $color; ?>;
                    }
                <?php
            }
            ?>
            
        </style>
        <?php if(isset($_GET['post_type']) && $_GET['post_type'] == 'shop_order'  ): 
            remove_filter('wc_order_statuses', array($this, 'woocommerce_status_actions_wc_order_statuses'), 10, 1);
            $order_statuses = wc_get_order_statuses();
            add_filter('wc_order_statuses', array($this, 'woocommerce_status_actions_wc_order_statuses'), 10, 1);
        ?>
            <script>
            jQuery(document).ready(function($) {
                <?php 
                if($modifed_def_st && is_array($modifed_def_st)){
                    foreach ($modifed_def_st as $slug => $val) {
                        if(!empty($val['name'])&& isset($order_statuses[$slug])){
                        ?>
                        var _a = $('ul.subsubsub li.<?php echo $slug; ?> a');
                            if(_a.length){
                                var html = _a.html();
                                var new_n = '<?php echo $val["name"]; ?>'.trim();

                                    html = html.replace( <?php echo '/'.str_replace('/', '', $order_statuses[$slug] ).'/i'; ?>, new_n);
                                    _a.html(html);
                            }
                        <?php
                        }
                    }
                }
                ?>
            });
            </script>
        <?php
        endif;
    }

}
}


/**
 * Returns the main instance of WoocommerceStatusActions to prevent the need to use globals.
 *
 * @since  1.4.9
 * @return WoocommerceStatusActions
 */
function WC_SA() {
    return WoocommerceStatusActions::instance();
}
global $wpdb;
$wpdb->wc_osmeta = $wpdb->prefix . "wc_osmeta";
$wpdb->wc_osa    = $wpdb->prefix . "woocommerce_order_status_action";
$GLOBALS['woocommercestatusactions'] = WC_SA();

if( !function_exists('wc_get_order_statuses') && version_compare(WC_SA()->wc_sa_get_woo_version(), '2.2', '<') ){
    function wc_get_order_statuses() {
        $order_statuses = array(
            'pending'    => _x( 'Pending Payment', 'Order status', 'woocommerce' ),
            'processing' => _x( 'Processing', 'Order status', 'woocommerce' ),
            'on-hold'    => _x( 'On Hold', 'Order status', 'woocommerce' ),
            'completed'  => _x( 'Completed', 'Order status', 'woocommerce' ),
            'cancelled'  => _x( 'Cancelled', 'Order status', 'woocommerce' ),
            'refunded'   => _x( 'Refunded', 'Order status', 'woocommerce' ),
            'failed'     => _x( 'Failed', 'Order status', 'woocommerce' ),
        );
        return apply_filters( 'wc_order_statuses', $order_statuses );
    }
}
if( !function_exists('wc_get_custom_status')  ){
    function wc_get_custom_status($order_status = '') {
        return new WC_Custom_Status($order_status);
    }
}
function woocommerce_status_actions_run_automatic_trigger($order_id, $new_status, $old_status)
    {
        $order  = wc_get_order($order_id);
        $all_st = wc_get_order_statuses();
        $new_status = 'wc-'.$new_status;
        if ( isset( $all_st[$new_status]) && $order->get_status() == $old_status ) {
            $note = apply_filters('bulk_handler_custom_action_note', '', $new_status, $order );
            $note = apply_filters('automatic_trigger_handler_custom_action_note', $note, $new_status, $order );
            $order->update_status( $new_status, $note );
        }
    }
?>